@@ -21,6 +21,7 @@
 #include <gtk/gtk.h>
 
 #include "gf-application.h"
+#include "libaudio-device-selection/gf-audio-device-selection.h"
 #include "libautomount-manager/gsd-automount-manager.h"
 #include "libbluetooth-applet/gf-bluetooth-applet.h"
 #include "libdesktop-background/gf-desktop-background.h"
@@ -28,14 +29,20 @@
 #include "libend-session-dialog/gf-end-session-dialog.h"
 #include "libidle-monitor/flashback-idle-monitor.h"
 #include "libinput-sources/gf-input-sources.h"
+#include "libnotifications/gf-notifications.h"
 #include "libpolkit/flashback-polkit.h"
 #include "libpower-applet/gf-power-applet.h"
 #include "libscreencast/gf-screencast.h"
+#include "libscreensaver/gf-screensaver.h"
 #include "libscreenshot/gf-screenshot.h"
 #include "libshell/flashback-shell.h"
 #include "libsound-applet/gf-sound-applet.h"
 #include "libworkarounds/gf-workarounds.h"
 
+#ifdef WITH_LIBSTATUS_NOTIFIER
+#include "libwatcher/gf-watcher.h"
+#endif
+
 struct _GfApplication
 {
   GObject                 parent;
@@ -44,40 +51,33 @@ struct _GfApplication
 
   GSettings              *settings;
 
-  GtkCssProvider         *provider;
+  GtkStyleProvider       *provider;
 
   GsdAutomountManager    *automount;
   FlashbackDisplayConfig *config;
   FlashbackIdleMonitor   *idle_monitor;
   FlashbackPolkit        *polkit;
   FlashbackShell         *shell;
+  GfAudioDeviceSelection *audio_device_selection;
   GfBluetoothApplet      *bluetooth;
   GfDesktopBackground    *background;
   GfEndSessionDialog     *dialog;
   GfInputSources         *input_sources;
+  GfNotifications        *notifications;
   GfPowerApplet          *power;
   GfScreencast           *screencast;
+  GfScreensaver          *screensaver;
   GfScreenshot           *screenshot;
   GfSoundApplet          *sound;
   GfWorkarounds          *workarounds;
+
+#ifdef WITH_LIBSTATUS_NOTIFIER
+  GfWatcher              *watcher;
+#endif
 };
 
 G_DEFINE_TYPE (GfApplication, gf_application, G_TYPE_OBJECT)
 
-static void
-remove_style_provider (GfApplication *application,
-                       GdkScreen     *screen)
-{
-  GtkStyleProvider *provider;
-
-  if (application->provider == NULL)
-    return;
-
-  provider = GTK_STYLE_PROVIDER (application->provider);
-  gtk_style_context_remove_provider_for_screen (screen, provider);
-  g_clear_object (&application->provider);
-}
-
 static void
 theme_changed (GtkSettings *settings,
                GParamSpec  *pspec,
@@ -85,34 +85,47 @@ theme_changed (GtkSettings *settings,
 {
   GfApplication *application;
   GdkScreen *screen;
-  gchar *theme;
+  gchar *theme_name;
+  gboolean dark_theme;
+  guint priority;
+  gchar *resource;
+  GtkCssProvider *css;
 
   application = GF_APPLICATION (user_data);
   screen = gdk_screen_get_default ();
 
-  g_object_get (settings, "gtk-theme-name", &theme, NULL);
-
-  remove_style_provider (application, screen);
-
-  if (g_strcmp0 (theme, "Adwaita") == 0 || g_strcmp0 (theme, "HighContrast") == 0)
+  if (application->provider != NULL)
     {
-      gchar *resource;
-      GtkStyleProvider *provider;
-      gint priority;
-
-      application->provider = gtk_css_provider_new ();
-
-      resource = g_strdup_printf ("/org/gnome/gnome-flashback/%s.css", theme);
-      gtk_css_provider_load_from_resource (application->provider, resource);
-      g_free (resource);
-
-      provider = GTK_STYLE_PROVIDER (application->provider);
-      priority = GTK_STYLE_PROVIDER_PRIORITY_APPLICATION;
-
-      gtk_style_context_add_provider_for_screen (screen, provider, priority);
+      gtk_style_context_remove_provider_for_screen (screen, application->provider);
+      g_clear_object (&application->provider);
     }
 
-  g_free (theme);
+  g_object_get (settings, "gtk-theme-name", &theme_name, NULL);
+
+  if (g_strcmp0 (theme_name, "Adwaita") != 0 &&
+      g_strcmp0 (theme_name, "HighContrast") != 0)
+    {
+      g_free (theme_name);
+      return;
+    }
+
+  g_object_get (settings,
+                "gtk-application-prefer-dark-theme", &dark_theme,
+                NULL);
+
+  priority = GTK_STYLE_PROVIDER_PRIORITY_APPLICATION;
+  resource = g_strdup_printf ("/org/gnome/gnome-flashback/theme/%s/gnome-flashback%s.css",
+                              theme_name, dark_theme ? "-dark" : "");
+
+  css = gtk_css_provider_new ();
+  application->provider =  GTK_STYLE_PROVIDER (css);
+
+  gtk_css_provider_load_from_resource (css, resource);
+  gtk_style_context_add_provider_for_screen (screen, application->provider,
+                                             priority);
+
+  g_free (theme_name);
+  g_free (resource);
 }
 
 static void
@@ -143,16 +156,23 @@ settings_changed (GSettings   *settings,
   SETTING_CHANGED (idle_monitor, "idle-monitor", flashback_idle_monitor_new)
   SETTING_CHANGED (polkit, "polkit", flashback_polkit_new)
   SETTING_CHANGED (shell, "shell", flashback_shell_new)
+  SETTING_CHANGED (audio_device_selection, "audio-device-selection", gf_audio_device_selection_new)
   SETTING_CHANGED (bluetooth, "bluetooth-applet", gf_bluetooth_applet_new)
   SETTING_CHANGED (background, "desktop-background", gf_desktop_background_new)
   SETTING_CHANGED (dialog, "end-session-dialog", gf_end_session_dialog_new)
   SETTING_CHANGED (input_sources, "input-sources", gf_input_sources_new)
+  SETTING_CHANGED (notifications, "notifications", gf_notifications_new)
   SETTING_CHANGED (power, "power-applet", gf_power_applet_new)
   SETTING_CHANGED (screencast, "screencast", gf_screencast_new)
+  SETTING_CHANGED (screensaver, "screensaver", gf_screensaver_new)
   SETTING_CHANGED (screenshot, "screenshot", gf_screenshot_new)
   SETTING_CHANGED (sound, "sound-applet", gf_sound_applet_new)
   SETTING_CHANGED (workarounds, "workarounds", gf_workarounds_new)
 
+#ifdef WITH_LIBSTATUS_NOTIFIER
+  SETTING_CHANGED (watcher, "watcher", gf_watcher_new)
+#endif
+
 #undef SETTING_CHANGED
 
   if (application->shell)
@@ -163,10 +183,8 @@ static void
 gf_application_dispose (GObject *object)
 {
   GfApplication *application;
-  GdkScreen *screen;
 
   application = GF_APPLICATION (object);
-  screen = gdk_screen_get_default ();
 
   if (application->bus_name)
     {
@@ -176,23 +194,30 @@ gf_application_dispose (GObject *object)
 
   g_clear_object (&application->settings);
 
-  remove_style_provider (application, screen);
+  g_clear_object (&application->provider);
 
   g_clear_object (&application->automount);
   g_clear_object (&application->config);
   g_clear_object (&application->idle_monitor);
   g_clear_object (&application->polkit);
   g_clear_object (&application->shell);
+  g_clear_object (&application->audio_device_selection);
   g_clear_object (&application->bluetooth);
   g_clear_object (&application->background);
   g_clear_object (&application->dialog);
   g_clear_object (&application->input_sources);
+  g_clear_object (&application->notifications);
   g_clear_object (&application->power);
   g_clear_object (&application->screencast);
   g_clear_object (&application->screenshot);
+  g_clear_object (&application->screensaver);
   g_clear_object (&application->sound);
   g_clear_object (&application->workarounds);
 
+#ifdef WITH_LIBSTATUS_NOTIFIER
+  g_clear_object (&application->watcher);
+#endif
+
   G_OBJECT_CLASS (gf_application_parent_class)->dispose (object);
 }
 
