@@ -786,6 +786,71 @@ compare_outputs (const void *one,
   return strcmp (o_one->name, o_two->name);
 }
 
+static void
+output_get_modes (FlashbackMonitorManager *manager,
+                  MetaOutput              *meta_output,
+                  XRROutputInfo           *output)
+{
+  guint j, k;
+  guint n_actual_modes;
+
+  meta_output->modes = g_new0 (MetaMonitorMode *, output->nmode);
+
+  n_actual_modes = 0;
+  for (j = 0; j < (guint)output->nmode; j++)
+    {
+      for (k = 0; k < manager->n_modes; k++)
+        {
+          if (output->modes[j] == (XID)manager->modes[k].mode_id)
+            {
+              meta_output->modes[n_actual_modes] = &manager->modes[k];
+              n_actual_modes += 1;
+              break;
+            }
+        }
+    }
+  meta_output->n_modes = n_actual_modes;
+  if (n_actual_modes > 0)
+    meta_output->preferred_mode = meta_output->modes[0];
+}
+
+static void
+output_get_crtcs (FlashbackMonitorManager *manager,
+                  MetaOutput              *meta_output,
+                  XRROutputInfo           *output)
+{
+  guint j, k;
+  guint n_actual_crtcs;
+
+  meta_output->possible_crtcs = g_new0 (MetaCRTC *, output->ncrtc);
+
+  n_actual_crtcs = 0;
+  for (j = 0; j < (unsigned)output->ncrtc; j++)
+    {
+      for (k = 0; k < manager->n_crtcs; k++)
+        {
+          if ((XID)manager->crtcs[k].crtc_id == output->crtcs[j])
+            {
+              meta_output->possible_crtcs[n_actual_crtcs] = &manager->crtcs[k];
+              n_actual_crtcs += 1;
+              break;
+            }
+        }
+    }
+
+  meta_output->n_possible_crtcs = n_actual_crtcs;
+
+  meta_output->crtc = NULL;
+  for (j = 0; j < manager->n_crtcs; j++)
+    {
+      if ((XID)manager->crtcs[j].crtc_id == output->crtc)
+        {
+          meta_output->crtc = &manager->crtcs[j];
+          break;
+        }
+    }
+}
+
 static char *
 get_xmode_name (XRRModeInfo *xmode)
 {
@@ -911,7 +976,7 @@ make_logical_config (FlashbackMonitorManager *manager)
   unsigned int i, j;
 
   monitor_infos = g_array_sized_new (FALSE, TRUE, sizeof (MetaMonitorInfo),
-                                     manager->n_outputs);
+                                     manager->n_crtcs);
 
   /* Walk the list of MetaCRTCs, and build a MetaMonitorInfo
      for each of them, unless they reference a rectangle that
@@ -1018,6 +1083,23 @@ make_logical_config (FlashbackMonitorManager *manager)
     add_monitor (manager, &manager->monitor_infos[i]);
 }
 
+static void
+clear_output (MetaOutput *output)
+{
+  g_free (output->name);
+  g_free (output->vendor);
+  g_free (output->product);
+  g_free (output->serial);
+  g_free (output->modes);
+  g_free (output->possible_crtcs);
+  g_free (output->possible_clones);
+
+  if (output->driver_notify)
+    output->driver_notify (output);
+
+  memset (output, 0, sizeof (*output));
+}
+
 static void
 free_output_array (MetaOutput *old_outputs,
                    int         n_old_outputs)
@@ -1025,22 +1107,22 @@ free_output_array (MetaOutput *old_outputs,
   int i;
 
   for (i = 0; i < n_old_outputs; i++)
-    {
-      g_free (old_outputs[i].name);
-      g_free (old_outputs[i].vendor);
-      g_free (old_outputs[i].product);
-      g_free (old_outputs[i].serial);
-      g_free (old_outputs[i].modes);
-      g_free (old_outputs[i].possible_crtcs);
-      g_free (old_outputs[i].possible_clones);
-
-      if (old_outputs[i].driver_notify)
-        old_outputs[i].driver_notify (&old_outputs[i]);
-    }
+    clear_output (&old_outputs[i]);
 
   g_free (old_outputs);
 }
 
+static void
+clear_mode (MetaMonitorMode *mode)
+{
+  g_free (mode->name);
+
+  if (mode->driver_notify)
+    mode->driver_notify (mode);
+
+  memset (mode, 0, sizeof (*mode));
+}
+
 static void
 free_mode_array (MetaMonitorMode *old_modes,
                  int              n_old_modes)
@@ -1048,16 +1130,29 @@ free_mode_array (MetaMonitorMode *old_modes,
   int i;
 
   for (i = 0; i < n_old_modes; i++)
-    {
-      g_free (old_modes[i].name);
-
-      if (old_modes[i].driver_notify)
-        old_modes[i].driver_notify (&old_modes[i]);
-    }
+    clear_mode (&old_modes[i]);
 
   g_free (old_modes);
 }
 
+static void
+clear_crtc (MetaCRTC *crtc)
+{
+  memset (crtc, 0, sizeof (*crtc));
+}
+
+static void
+free_crtc_array (MetaCRTC *old_crtcs,
+                 int       n_old_crtcs)
+{
+  int i;
+
+  for (i = 0; i < n_old_crtcs; i++)
+    clear_crtc (&old_crtcs[i]);
+
+  g_free (old_crtcs);
+}
+
 static guint8 *
 get_edid_property (Display  *dpy,
                    RROutput  output,
@@ -1260,10 +1355,12 @@ read_current_config (FlashbackMonitorManager *manager)
       MetaOutput *meta_output;
 
       output = XRRGetOutputInfo (priv->xdisplay, resources, resources->outputs[i]);
+      if (!output)
+        continue;
 
       meta_output = &manager->outputs[n_actual_outputs];
 
-      if (output->connection != RR_Disconnected && output->nmode != 0)
+      if (output->connection != RR_Disconnected)
         {
           GBytes *edid;
 
@@ -1282,49 +1379,8 @@ read_current_config (FlashbackMonitorManager *manager)
           meta_output->connector_type = output_get_connector_type (priv, meta_output);
 
           output_get_tile_info (priv, meta_output);
-
-          meta_output->n_modes = output->nmode;
-          meta_output->modes = g_new0 (MetaMonitorMode *, meta_output->n_modes);
-
-          for (j = 0; j < meta_output->n_modes; j++)
-            {
-              for (k = 0; k < manager->n_modes; k++)
-                {
-                  if (output->modes[j] == (XID)manager->modes[k].mode_id)
-                    {
-                      meta_output->modes[j] = &manager->modes[k];
-                      break;
-                    }
-                }
-            }
-
-          meta_output->preferred_mode = meta_output->modes[0];
-
-          meta_output->n_possible_crtcs = output->ncrtc;
-          meta_output->possible_crtcs = g_new0 (MetaCRTC *, meta_output->n_possible_crtcs);
-
-          for (j = 0; j < (unsigned)output->ncrtc; j++)
-            {
-              for (k = 0; k < manager->n_crtcs; k++)
-                {
-                  if ((XID)manager->crtcs[k].crtc_id == output->crtcs[j])
-                    {
-                      meta_output->possible_crtcs[j] = &manager->crtcs[k];
-                      break;
-                    }
-                }
-            }
-
-          meta_output->crtc = NULL;
-
-          for (j = 0; j < manager->n_crtcs; j++)
-            {
-              if ((XID)manager->crtcs[j].crtc_id == output->crtc)
-                {
-                  meta_output->crtc = &manager->crtcs[j];
-                  break;
-                }
-            }
+          output_get_modes (manager, meta_output, output);
+          output_get_crtcs (manager, meta_output, output);
 
           meta_output->n_possible_clones = output->nclone;
           meta_output->possible_clones = g_new0 (MetaOutput *, meta_output->n_possible_clones);
@@ -1348,7 +1404,10 @@ read_current_config (FlashbackMonitorManager *manager)
         else
           meta_output->backlight = -1;
 
-        n_actual_outputs++;
+        if (meta_output->n_modes == 0 || meta_output->n_possible_crtcs == 0)
+          clear_output (meta_output);
+        else
+          n_actual_outputs++;
       }
 
       XRRFreeOutputInfo (output);
@@ -1473,8 +1532,8 @@ flashback_monitor_manager_finalize (GObject *object)
 
   free_output_array (manager->outputs, manager->n_outputs);
   free_mode_array (manager->modes, manager->n_modes);
+  free_crtc_array (manager->crtcs, manager->n_crtcs);
   g_free (manager->monitor_infos);
-  g_free (manager->crtcs);
 
   G_OBJECT_CLASS (flashback_monitor_manager_parent_class)->finalize (object);
 }
@@ -1949,6 +2008,7 @@ flashback_monitor_manager_read_current_config (FlashbackMonitorManager *manager)
   MetaCRTC *old_crtcs;
   MetaMonitorMode *old_modes;
   unsigned int n_old_outputs;
+  unsigned int n_old_crtcs;
   unsigned int n_old_modes;
 
   /* Some implementations of read_current use the existing information
@@ -1959,6 +2019,7 @@ flashback_monitor_manager_read_current_config (FlashbackMonitorManager *manager)
   old_modes = manager->modes;
   n_old_modes = manager->n_modes;
   old_crtcs = manager->crtcs;
+  n_old_crtcs = manager->n_crtcs;
 
   manager->serial++;
 
@@ -1966,7 +2027,7 @@ flashback_monitor_manager_read_current_config (FlashbackMonitorManager *manager)
 
   free_output_array (old_outputs, n_old_outputs);
   free_mode_array (old_modes, n_old_modes);
-  g_free (old_crtcs);
+  free_crtc_array (old_crtcs, n_old_crtcs);
 }
 
 void
